@@ -400,7 +400,8 @@ class _BatchProcessorMixin:
         processor is being added to the database.
 
         If this processor is not already scheduled to run, this will schedule
-        it.
+        it.  It will also start the batch process if it is not yet running and
+        there are any registered remote listeners.
         """
         localCount = self.store.query(
             _ReliableListener,
@@ -408,9 +409,19 @@ class _BatchProcessorMixin:
                            _ReliableListener.style == iaxiom.LOCAL),
             limit=1).count()
 
+        remoteCount = self.store.query(
+            _ReliableListener,
+            attributes.AND(_ReliableListener.processor == self,
+                           _ReliableListener.style == iaxiom.REMOTE),
+            limit=1).count()
+
         if localCount and self.scheduled is None:
             self.scheduled = extime.Time()
             iaxiom.IScheduler(self.store).schedule(self, self.scheduled)
+        if remoteCount:
+            batchService = iaxiom.IBatchService(self.store, None)
+            if batchService is not None:
+                batchService.start()
 
 
 
@@ -648,7 +659,6 @@ class ProcessController(object):
     def _startProcess(self):
         executable = sys.executable
         env = os.environ
-        env['PYTHONPATH'] = os.pathsep.join(sys.path)
 
         twistdBinaries = procutils.which("twistd2.4") + procutils.which("twistd")
         if not twistdBinaries:
@@ -895,7 +905,13 @@ class BatchProcessingControllerService(service.Service):
     """
     Controls starting, stopping, and passing messages to the system process in
     charge of remote batch processing.
+
+    @ivar batchController: A reference to the L{ProcessController} for
+        interacting with the batch process, if one exists.  Otherwise C{None}.
     """
+    implements(iaxiom.IBatchService)
+
+    batchController = None
 
     def __init__(self, store):
         self.store = store
@@ -949,6 +965,11 @@ class BatchProcessingControllerService(service.Service):
                            method=method).do)
 
 
+    def start(self):
+        if self.batchController is not None:
+            self.batchController.getProcess()
+
+
     def suspend(self, storepath, storeID):
         return self.batchController.getProcess().addCallback(
             SuspendProcessor(storepath=storepath, storeid=storeID).do)
@@ -978,6 +999,10 @@ class _SubStoreBatchChannel(object):
         return self.service.call(itemMethod)
 
 
+    def start(self):
+        self.service.start()
+
+
     def suspend(self, storeID):
         return self.service.suspend(self.storepath, storeID)
 
@@ -988,9 +1013,23 @@ class _SubStoreBatchChannel(object):
 
 
 def storeBatchServiceSpecialCase(st, pups):
+    """
+    Adapt a L{Store} to L{IBatchService}.
+
+    If C{st} is a substore, return a simple wrapper that delegates to the site
+    store's L{IBatchService} powerup.  Return C{None} if C{st} has no
+    L{BatchProcessingControllerService}.
+    """
     if st.parent is not None:
-        return _SubStoreBatchChannel(st)
-    return service.IService(st).getServiceNamed("Batch Processing Controller")
+        try:
+            return _SubStoreBatchChannel(st)
+        except TypeError:
+            return None
+    storeService = service.IService(st)
+    try:
+        return storeService.getServiceNamed("Batch Processing Controller")
+    except KeyError:
+        return None
 
 
 
