@@ -10,7 +10,6 @@ from weakref import WeakValueDictionary
 
 from twisted.python import log
 from twisted.python.reflect import qual, namedAny
-from twisted.python.util import unsignedID
 from twisted.python.util import mergeFunctionMetadata
 from twisted.application.service import IService, IServiceCollection, MultiService
 
@@ -62,7 +61,7 @@ class MetaItem(slotmachine.SchemaMetaMachine):
             T.typeName = normalize(qual(T))
         if T.schemaVersion is None:
             T.schemaVersion = 1
-        if T.typeName in _typeNameToMostRecentClass:
+        if not T.__legacy__ and T.typeName in _typeNameToMostRecentClass:
             # Let's try not to gc.collect() every time.
             gc.collect()
         if T.typeName in _typeNameToMostRecentClass:
@@ -193,7 +192,6 @@ class Empowered(object):
         return value is the powerup.  These are used only by the callable
         interface adaption API, not C{powerupsFor}.
     """
-
     aggregateInterfaces = {
         IService: serviceSpecialCase,
         IServiceCollection: serviceSpecialCase}
@@ -314,19 +312,22 @@ class Empowered(object):
         special rules.  The full list of such interfaces is present in the
         'aggregateInterfaces' class attribute.
         """
-        if (self.store is None  # Don't bother doing a *query* if we're not
-                                # even stored in a store yet
-            or interface is IPowerupIndirector): # you can't do a query for
-                                                 # IPowerupIndirector, that
-                                                 # would just start an infinite
-                                                 # loop.
+        if interface is IPowerupIndirector:
+            # This would cause an infinite loop, since powerupsFor will try to
+            # adapt every powerup to IPowerupIndirector, calling this method.
             return
-        pups = self.powerupsFor(interface)
-        agg = self.aggregateInterfaces
-        if interface in agg:
-            return agg[interface](self, pups)
-        for p in pups:
-            return p
+
+        try:
+            pups = self.powerupsFor(interface)
+        except AttributeError:  # self.store is None -> self.store.query...
+            return
+
+        aggregator = self.aggregateInterfaces.get(interface, None)
+        if aggregator is not None:
+            return aggregator(self, pups)
+
+        for pup in pups:
+            return pup  # return first one, or None if no powerups
 
 
     def powerupsFor(self, interface):
@@ -561,23 +562,16 @@ class Item(Empowered, slotmachine._Strict):
 
     store = property(*store())
 
+
     def __repr__(self):
         """
         Return a nice string representation of the Item which contains some
         information about each of its attributes.
         """
-
-        L = [self.__name__]
-        L.append('(')
-        A = []
-        for nam, atr in sorted(self.getSchema()):
-            V = atr.reprFor(self)
-            A.append('%s=%s' % (nam, V))
-        A.append('storeID=' + str(self.storeID))
-        L.append(', '.join(A))
-        L.append(')')
-        L.append('@0x%X' % unsignedID(self))
-        return ''.join(L)
+        attrs = ", ".join("{n}={v}".format(n=name, v=attr.reprFor(self))
+                          for name, attr in sorted(self.getSchema()))
+        template = b"{s.__name__}({attrs}, storeID={s.storeID})@{id:#x}"
+        return template.format(s=self, attrs=attrs, id=id(self))
 
 
     def __subinit__(self, **kw):
@@ -659,7 +653,7 @@ class Item(Empowered, slotmachine._Strict):
                          storeID=storeID,
                          __everInserted=True)
 
-        schema = list(self.getSchema())
+        schema = self.getSchema()
         assert len(schema) == len(attrs), "invalid number of attributes"
         for data, (name, attr) in zip(attrs, schema):
             attr.loaded(self, data)
@@ -675,11 +669,13 @@ class Item(Empowered, slotmachine._Strict):
         """
         return all persistent class attributes
         """
+        schema = []
         for name, atr in cls.__attributes__:
             atr = atr.__get__(None, cls)
             if isinstance(atr, SQLAttribute):
-                yield (name, atr)
-
+                schema.append((name, atr))
+        cls.getSchema = staticmethod(lambda schema=schema: schema)
+        return schema
     getSchema = classmethod(getSchema)
 
 
@@ -1069,9 +1065,14 @@ class Placeholder(object):
         # once in the "SELECT ..." part of the query, determined by
         # getSchema(). In this case, the correct placeholder names
         # need to be used.
+        schema = []
         for (name, atr) in self._placeholderItemClass.getSchema():
-            yield name, _PlaceholderColumn(self,
-                                           getattr(self._placeholderItemClass, name))
+            schema.append((
+                    name,
+                    _PlaceholderColumn(
+                        self, getattr(self._placeholderItemClass, name))))
+        return schema
+
 
     def getTableName(self, store):
         return self._placeholderItemClass.getTableName(store)
@@ -1136,4 +1137,3 @@ class _PowerupConnector(Item):
 
 POWERUP_BEFORE = 1              # Priority for 'high' priority powerups.
 POWERUP_AFTER = -1              # Priority for 'low' priority powerups.
-
