@@ -4,7 +4,6 @@
 """
 This module holds the Axiom Store class and related classes, such as queries.
 """
-
 from epsilon import hotfix
 hotfix.require('twisted', 'filepath_copyTo')
 
@@ -17,7 +16,6 @@ from twisted.python.failure import Failure
 from twisted.python import filepath
 from twisted.internet import defer
 from twisted.python.reflect import namedAny
-from twisted.python.util import unsignedID
 from twisted.application.service import IService, IServiceCollection
 
 from epsilon.pending import PendingEvent
@@ -140,6 +138,13 @@ def storeServiceSpecialCase(st, pups):
         batcher = batch.BatchProcessingControllerService(st)
         batcher.setServiceParent(collection)
 
+    scheduler = iaxiom.IScheduler(st)
+    # If it's an old database, we might get a SubScheduler instance.  It has no
+    # setServiceParent method.
+    setServiceParent = getattr(scheduler, 'setServiceParent', None)
+    if setServiceParent is not None:
+        setServiceParent(collection)
+
     return collection
 
 
@@ -989,7 +994,6 @@ def _schedulerServiceSpecialCase(empowered, pups):
                 sched = _SiteScheduler(empowered)
             else:
                 sched = _UserScheduler(empowered)
-            sched.setServiceParent(IService(empowered))
             empowered._schedulerService = sched
         return empowered._schedulerService
     return None
@@ -1338,12 +1342,12 @@ class Store(Empowered):
 
 
     def __repr__(self):
-        d = self.dbdir
-        if d is None:
-            d = '(in memory)'
-        else:
-            d = repr(d)
-        return '<Store %s@0x%x>' % (d, unsignedID(self))
+        dbdir = self.dbdir
+        if self.dbdir is None:
+            dbdir = '(in memory)'
+
+        return "<Store {dbdir}@{id:#x}".format(dbdir=dbdir, id=id(self))
+
 
     def findOrCreate(self, userItemClass, __ifnew=None, **attrs):
         """
@@ -1522,8 +1526,8 @@ class Store(Empowered):
 
         if (key[0], key[1] + 1) in onDiskSchema:
             raise RuntimeError(
-                "Greater versions of database %r objects in the DB than in memory" %
-                (actualType.typeName,))
+                "Memory version of %r is %d; database has newer" % (
+                    actualType.typeName, key[1]))
 
 
     # finally find old versions of the data and prepare to upgrade it.
@@ -1741,10 +1745,10 @@ class Store(Empowered):
             self.executeSQL(sql, insertArgs)
 
     def _loadedItem(self, itemClass, storeID, attrs):
-        if self.objectCache.has(storeID):
+        try:
             result = self.objectCache.get(storeID)
             # XXX do checks on consistency between attrs and DB object, maybe?
-        else:
+        except KeyError:
             result = itemClass.existingInStore(self, storeID, attrs)
             if not result.__legacy__:
                 self.objectCache.cache(storeID, result)
@@ -2199,8 +2203,10 @@ class Store(Empowered):
                     type(storeID).__name__,))
         if storeID == STORE_SELF_ID:
             return self
-        if self.objectCache.has(storeID):
+        try:
             return self.objectCache.get(storeID)
+        except KeyError:
+            pass
         log.msg(interface=iaxiom.IStatEvent, stat_cache_misses=1, key=storeID)
         results = self.querySchemaSQL(_schema.TYPEOF_QUERY, [storeID])
         assert (len(results) in [1, 0]),\
