@@ -1,14 +1,18 @@
+# Copyright 2008 Divmod, Inc.  See LICENSE for details
+
 import sys
 import os
 
 from twisted.trial import unittest
 from twisted.internet import protocol, defer
 from twisted.python.util import sibpath
+from twisted.python import log, filepath
 
 from epsilon import extime
 from axiom import attributes, item, store, errors
+from axiom.iaxiom import IStatEvent
 
-from pysqlite2.dbapi2 import sqlite_version_info
+from axiom._pysqlite2 import sqlite_version_info
 
 
 class RevertException(Exception):
@@ -44,13 +48,13 @@ class TestItem(item.Item):
 
 class StoreTests(unittest.TestCase):
     def testCreation(self):
-        dbdir = self.mktemp()
+        dbdir = filepath.FilePath(self.mktemp())
         s = store.Store(dbdir)
         s.close()
 
 
     def testReCreation(self):
-        dbdir = self.mktemp()
+        dbdir = filepath.FilePath(self.mktemp())
         s = store.Store(dbdir)
         s.close()
         s = store.Store(dbdir)
@@ -62,7 +66,7 @@ class StoreTests(unittest.TestCase):
         A Store should raise an error if both dbdir and filesdir are specified.
         """
         self.assertRaises(ValueError, store.Store,
-                          self.mktemp(), filesdir=self.mktemp())
+                          filepath.FilePath(self.mktemp()), filesdir=filepath.FilePath(self.mktemp()))
 
 
     def testTableQueryCaching(self):
@@ -117,6 +121,7 @@ class StoreTests(unittest.TestCase):
         s = store.Store()
         self.assertRaises(errors.ItemClassesOnly, s.getTableName, TestItem(store=s))
 
+
     def testTableNameCacheDoesntGrow(self):
         """
         Make sure the table name cache doesn't grow out of control anymore.
@@ -127,13 +132,270 @@ class StoreTests(unittest.TestCase):
         for i in range(10):
             s.getTableName(TestItem)
         self.assertEquals(x, len(s.typeToTableNameCache))
+
+
     def testStoreIDComparerIdentity(self):
         # We really want this to hold, because the storeID object is
         # used like a regular attribute as a key for various caching
         # within store.
         a0 = TestItem.storeID
         a1 = TestItem.storeID
-        assert a0 is a1
+        self.assertIdentical(a0, a1)
+
+
+    def test_loadTypeSchema(self):
+        """
+        L{Store._loadTypeSchema} returns a C{dict} mapping item type names and
+        versions to a list of tuples giving information about the on-disk
+        schema information for each attribute of that version of that type.
+        """
+        s = store.Store()
+        TestItem(store=s)
+
+        loadedSchema = s._loadTypeSchema()
+        self.assertEqual(
+            loadedSchema[(TestItem.typeName, TestItem.schemaVersion)],
+            [('bar', 'TEXT COLLATE NOCASE', False, attributes.text, ''),
+             ('baz', 'INTEGER', False, attributes.timestamp, ''),
+             ('booleanF', 'BOOLEAN', False, attributes.boolean, ''),
+             ('booleanT', 'BOOLEAN', False, attributes.boolean, ''),
+             ('foo', 'INTEGER', True, attributes.integer, ''),
+             ('myStore', 'INTEGER', True, attributes.reference, ''),
+             ('other', 'INTEGER', True, attributes.reference, ''),
+             ])
+
+
+    def test_checkInconsistentTypeSchema(self):
+        """
+        L{Store._checkTypeSchemaConsistency} raises L{RuntimeError} if the in
+        memory schema of the type passed to it differs from the schema stored
+        in the database for that type, either by including too few attributes,
+        too many attributes, or the wrong type for one of the attributes.
+        """
+        s = store.Store()
+
+        schema = [
+            (name, attr.sqltype, attr.indexed, attr, attr.doc)
+            for (name, attr) in TestItem.getSchema()]
+
+        # Test a missing attribute
+        self.assertRaises(
+            RuntimeError,
+            s._checkTypeSchemaConsistency,
+            TestItem,
+            {(TestItem.typeName, TestItem.schemaVersion): schema[:-1]})
+
+        # And an extra attribute
+        self.assertRaises(
+            RuntimeError,
+            s._checkTypeSchemaConsistency,
+            TestItem,
+            {(TestItem.typeName, TestItem.schemaVersion):
+                 schema + [schema[0]]})
+
+        # And the wrong type for one of the attributes
+        self.assertRaises(
+            RuntimeError,
+            s._checkTypeSchemaConsistency,
+            TestItem,
+            {(TestItem.typeName, TestItem.schemaVersion):
+                 [(schema[0], 'VARCHAR(64) (this is made up)',
+                   schema[2], schema[3], schema[4])] + schema[1:]})
+
+
+    def test_checkOutdatedTypeSchema(self):
+        """
+        L{Store._checkTypeSchemaConsistency} raises L{RuntimeError} if the type
+        passed to it is the most recent in-memory version of that type and is
+        older than the newest on disk schema for that type.
+        """
+        s = store.Store()
+
+        schema = [
+            (name, attr.sqltype, attr.indexed, attr, attr.doc)
+            for (name, attr) in TestItem.getSchema()]
+
+        self.assertRaises(
+            RuntimeError,
+            s._checkTypeSchemaConsistency,
+            TestItem,
+            {(TestItem.typeName, TestItem.schemaVersion): schema,
+             (TestItem.typeName, TestItem.schemaVersion + 1): schema})
+
+
+    def test_checkConsistencyWhenOpened(self):
+        """
+        L{Store.__init__} checks the consistency of the schema and raises
+        L{RuntimeError} for any inconsistency.
+        """
+        class SoonToChange(item.Item):
+            attribute = attributes.integer()
+
+        dbpath = self.mktemp()
+        s = store.Store(dbpath)
+        SoonToChange(store=s)
+        s.close()
+
+        # Get rid of the cached information about this type
+        from axiom.item import _typeNameToMostRecentClass
+        del _typeNameToMostRecentClass[SoonToChange.typeName]
+        del SoonToChange
+
+        class SoonToChange(item.Item):
+            attribute = attributes.boolean()
+
+        self.assertRaises(RuntimeError, store.Store, dbpath)
+
+
+    def test_createAndLoadExistingIndexes(self):
+        """
+        L{Store._loadExistingIndexes} returns a C{set} containing the names of
+        all of the indexes which exist in the store.
+        """
+        s = store.Store()
+        before = s._loadExistingIndexes()
+        TestItem(store=s)
+        after = s._loadExistingIndexes()
+        created = after - before
+
+        self.assertEqual(
+            created,
+            set([s._indexNameOf(TestItem, ['foo']),
+                 s._indexNameOf(TestItem, ['other']),
+                 s._indexNameOf(TestItem, ['myStore']),
+                 s._indexNameOf(TestItem, ['bar', 'baz'])]))
+
+
+    def test_loadExistingAttachedStoreIndexes(self):
+        """
+        If a store is attached to its parent, L{Store._loadExistingIndexes}
+        returns just the indexes which exist in the child store.
+        """
+        secondaryPath = self.mktemp()
+        main = store.Store()
+        secondary = store.Store(secondaryPath, parent=main, idInParent=17)
+
+        TestItem(store=secondary)
+        before = secondary._loadExistingIndexes()
+        secondary.attachToParent()
+        after = secondary._loadExistingIndexes()
+
+        self.assertEqual(before, after)
+
+
+    def test_createAttachedStoreIndexes(self):
+        """
+        Indexes created by the insertion of the first item of a type into a
+        store are created in the database which backs that store even if that
+        store is attached to another store backed by a different database.
+        """
+        secondaryPath = self.mktemp()
+
+        main = store.Store()
+        secondary = store.Store(secondaryPath, parent=main, idInParent=23)
+
+        before = secondary._loadExistingIndexes()
+        secondary.attachToParent()
+        TestItem(store=secondary)
+
+        # Close it to detach from the parent.  Detach from the parent so we can
+        # re-open and hopefully avoid accidentally getting any results polluted
+        # by the parent store (shouldn't happen, because the above test,
+        # test_loadExistingAttachedStoreIndexes, makes sure we can inspect
+        # indexes without worrying about an attached parent, but I'm being
+        # paranoid).
+        secondary.close()
+        del secondary
+
+        secondary = store.Store(secondaryPath, parent=main, idInParent=23)
+        after = secondary._loadExistingIndexes()
+
+        self.assertEqual(
+            after - before,
+            set([secondary._indexNameOf(TestItem, ['foo']),
+                 secondary._indexNameOf(TestItem, ['other']),
+                 secondary._indexNameOf(TestItem, ['myStore']),
+                 secondary._indexNameOf(TestItem, ['bar', 'baz'])]))
+
+
+    def test_loadPythonModuleHint(self):
+        """
+        If the Python definition of a type found in a Store has not yet been
+        loaded, the hint in the I{module} column in type table is loaded.
+        """
+        # Arbitrary constants used in multiple places and processes.
+        typeCount = 3
+        magicOffset = 17
+        baseModuleName = "axiom_unloaded_module_"
+
+        # Path the temporary new modules will be created in.
+        importPath = filepath.FilePath(self.mktemp())
+        importPath.makedirs()
+        sys.path.insert(0, importPath.path)
+        self.addCleanup(sys.path.remove, importPath.path)
+
+        # Path the store will be created at.
+        dbdir = filepath.FilePath(self.mktemp())
+
+        # Create some source files, each defining an item type.
+        for counter in range(typeCount):
+            moduleName = baseModuleName + str(counter)
+
+            # Sanity check - the test can't work if any of the modules is
+            # already imported.
+            self.assertNotIn(moduleName, sys.modules)
+
+            # Write out the source.
+            modulePath = importPath.child(moduleName + ".py")
+            modulePath.setContent("""\
+from axiom.item import Item
+from axiom.attributes import integer
+
+class Unloaded(Item):
+    value = integer()
+""")
+
+        # In another process, so as not to cause the unloaded modules to be
+        # loaded in this process, create a store containing instances of the
+        # Unloaded types.
+        script = filepath.FilePath(self.mktemp())
+        script.setContent("""\
+from sys import argv
+from twisted.python.reflect import namedAny
+from axiom.store import Store
+
+dbdir, typeCount, moduleBase, magicOffset = argv[1:]
+
+s = Store(dbdir)
+for i in range(int(typeCount)):
+    moduleName = moduleBase + str(i)
+    namedAny(moduleName).Unloaded(store=s, value=int(magicOffset) + i)
+s.close()
+""")
+
+        os.system(" ".join([
+                    "PYTHONPATH=%s:$PYTHONPATH" % (importPath.path,),
+                    sys.executable, script.path,
+                    dbdir.path, str(typeCount), baseModuleName,
+                    str(magicOffset)]))
+
+        # Another sanity check.  The modules still better not have been
+        # imported in this process.
+        for counter in range(typeCount):
+            self.assertNotIn(baseModuleName + str(counter), sys.modules)
+
+        # Now open the store here.  This only works if the Store figures out it
+        # needs to import the modules defining the types.
+        s = store.Store(dbdir.path)
+
+        # And to be sure, find the item and make sure it has the value we
+        # expect.
+        for counter in range(typeCount):
+            Unloaded = sys.modules[baseModuleName + str(counter)].Unloaded
+            self.assertEqual(
+                s.query(Unloaded,
+                        Unloaded.value == magicOffset + counter).count(), 1)
+
 
 
 class FailurePathTests(unittest.TestCase):
@@ -167,7 +429,7 @@ class FailurePathTests(unittest.TestCase):
 
 class ItemTests(unittest.TestCase):
     def setUp(self):
-        self.dbdir = self.mktemp()
+        self.dbdir = filepath.FilePath(self.mktemp())
         self.store = store.Store(self.dbdir)
 
     def tearDown(self):
@@ -296,6 +558,7 @@ class ItemTests(unittest.TestCase):
             self.fail("Transaction should have raised an exception")
 
 
+
 class AttributefulItem(item.Item):
     schemaVersion = 1
     typeName = 'test_attributeful_item'
@@ -303,9 +566,7 @@ class AttributefulItem(item.Item):
     withDefault = attributes.integer(default=42)
     withoutDefault = attributes.integer()
 
-    def __repr__(self):
-        return 'AttributeItem(oid=%s,withDefault=%s,withoutDefault=%s)'%  (
-            self.storeID, self.withDefault, self.withoutDefault)
+
 
 class StricterItem(item.Item):
     schemaVersion = 1
@@ -480,7 +741,7 @@ class DeleteFromStoreTrackingItem(item.Item):
 class MassInsertDeleteTests(unittest.TestCase):
 
     def setUp(self):
-        self.storepath = self.mktemp()
+        self.storepath = filepath.FilePath(self.mktemp())
         self.store = store.Store(self.storepath)
 
     def testBatchInsert(self):
@@ -667,11 +928,11 @@ class ProcessConcurrencyTestCase(unittest.TestCase,
             self.d.errback(reason)
 
     def testNewItemTypeInSubprocess(self):
-        dbdir = self.mktemp()
+        dbdir = filepath.FilePath(self.mktemp())
         self.store = store.Store(dbdir)
         # Open the store and leave its schema empty (don't create any items)
         # until the subprocess has opened it and loaded the bogus schema.
-        return self.spawn(sibpath(__file__, "openthenload.py"), dbdir)
+        return self.spawn(sibpath(__file__, "openthenload.py"), dbdir.path)
 
 
 class ConcurrencyTestCase(unittest.TestCase):
@@ -683,7 +944,7 @@ class ConcurrencyTestCase(unittest.TestCase):
         basically harmless and the query will work if re-executed.  This should
         be done transparently.
         """
-        dbdir = self.mktemp()
+        dbdir = filepath.FilePath(self.mktemp())
 
         firstStore = store.Store(dbdir)
         ConcurrentItemA(store=firstStore)
@@ -706,10 +967,47 @@ class ConcurrencyTestCase(unittest.TestCase):
         tracks that schema.  Test to make sure that creating the first instance
         of an Item subclass in one store is reflected in a second store.
         """
-        dbdir = self.mktemp()
+        dbdir = filepath.FilePath(self.mktemp())
 
         firstStore = store.Store(dbdir)
         secondStore = store.Store(dbdir)
 
         ConcurrentItemA(store=firstStore)
         self.assertEquals(secondStore.query(ConcurrentItemA).count(), 1)
+
+
+
+class LoggingTests(unittest.TestCase):
+    """
+    Tests for log events emitted by L{axiom.store}.
+    """
+    def _openTest(self, dbdir, expectedValue):
+        events = []
+        log.addObserver(events.append)
+        self.addCleanup(log.removeObserver, events.append)
+        store.Store(dbdir).close()
+        for event in events:
+            if event.get('interface') is not IStatEvent:
+                continue
+            if event.get('store_opened') != expectedValue:
+                continue
+            break
+        else:
+            self.fail("store_opened IStatEvent not emitted")
+
+
+    def test_openOnDisk(self):
+        """
+        Opening a file-backed store logs an event including the path to the
+        store.
+        """
+        dbdir = self.mktemp()
+        self._openTest(dbdir, os.path.abspath(dbdir))
+
+
+    def test_openInMemory(self):
+        """
+        Opening a memory-backed store logs an event with an empty string for
+        the path to the store.
+        """
+        self._openTest(None, '')
